//*****************************************************************************
// InterComm
// Version 1.0 Sep 2005
//
// 1.0 -> -Everything is new
//
// Sylvain Bissonnette
//*****************************************************************************
// Editor : UltraEdit32
//*****************************************************************************
//
//                    R E T U R N   S T A C K   3 2
//                          X T A L  16 MHZ
//                      BootLoader of 1024 word
//
//*****************************************************************************
//
//                          F U S E   B I T
//
//( )7      ( )6     ( )BL12 ( )BL11  ( )BL02   ( )BL01    ( )Lock2   ( )Lock1
//( )7      ( )6     ( )     ( )      ( )       ( )M103C   ( )WDTON   ( )
//( )OCDEN  ( )JTAGE (X)SPIEN(X)CKOPT (X)EESAVE ( )BOOTSZ1 (X)BOOTSZ0 (X)BOOTRST
//(X)BODLEV (X)BODEN ( )SUT1 (X)SUT0  ( )CKSEL3 (X)CKSEL2  ( )CKSEL1  ( )CKSEL0
//
//*****************************************************************************
//                          P I N   U S A G E
//
// PA0 -> GND
// PA1 -> A/D input from Microphone
// PA2 -> GND
// PA3 -> GND
// PA4 -> GND
// PA5 -> GND
// PA6 -> GND
// PA7 -> GND
//
// PB0 -> Keypad ROW1
// PB1 -> Keypad ROW2 
// PB2 -> Keypad ROW3
// PB3 -> Keypad ROW3
// PB4 -> Keypad COL1
// PB5 -> Keypad COL2
// PB6 -> Keypad COL3
// PB7 -> Keypad COL4
//
// PC0 -> LCD DB7
// PC1 -> LCD DB6
// PC2 -> LCD DB5
// PC3 -> LCD DB4
// PC4 -> LCD E
// PC5 -> LCD R/W
// PC6 -> LCD RS
// PC7 -> LCD BackLight
//
// PD0 -> Net_RX
// PD1 -> Net_TX
// PD2 -> +5
// PD3 -> LED
// PD4 -> PWM Volume
// PD5 -> PWM Voice
// PD6 -> Ampli Standby
// PD7 -> LCD Contrast
//
//*****************************************************************************
//                        T I M E R   U S A G E
//
// Timer 0 is use for 20khz sampling & reproducing voice
// Timer 1 is use for 62.5khz PWM voice reproducing and volume
// Timer 2 is use for LCD Contrast
// Timer 3 not use
//
//*****************************************************************************

//*****************************************************************************
//                            I N C L U D E
//*****************************************************************************
#include <iom32v.h>
#include <shortnametype.h>
#include <macros.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <eeprom.h>

//*****************************************************************************
//                            D E F I N E
//*****************************************************************************
#define VERSION                 10
#define TRUE                    1
#define FALSE                   0
#define XTAL                    16000000
#define VERSIONTEXT							"  InterComm  V:2.1\0"

#define VOICEBUFFER             64

#define OFF                     0
#define TALK                    1
#define LISTEN                  2
#define TONE                    3
#define BUSY                    4

#define PWM_VOICE               0x20
#define PWM_VOLUME              0x10
#define AMPLI_STANDBY           0x40

#define EEPROM_VOLUME           1
#define EEPROM_ADDRESS          2
#define EEPROM_KEYSOUND         3
#define EEPROM_KEYSOUNDVOLUME   4
#define EEPROM_KEYSOUNDTONE     5
#define EEPROM_KEYSOUNDLEN      6
#define EEPROM_BROADCAST        7

#define LED                     0x08

#define MAXITEM                 20

// Network
#define NET_DDR                 DDRD
#define NET_PIN                 PIND
#define NET_PORT                PORTD
#define NET_RX                  0x01
#define NET_TX                  0x02

#define NET_UBRRH               UBRRH
#define NET_UBRRL               UBRRL
#define NET_UCSRA               UCSRA
#define NET_UCSRB               UCSRB
#define NET_UCSRC               UCSRC
#define NET_UDR                 UDR

#define NET_SPEED               500000
#define NET_GET_ADDRESS         EEPROMread(EEPROM_ADDRESS);

// LCD
#define LCD_D7                  0x01
#define LCD_D6                  0x02
#define LCD_D5                  0x04
#define LCD_D4                  0x08
#define LCD_E                   0x10
#define LCD_RW                  0x20
#define LCD_RS                  0x40
#define LCD_BACKLIGHT           0x80
                                
#define LCD_PIN                 PINC
#define LCD_DDR                 DDRC
#define LCD_PORT                PORTC
                                
#define LCD_DATA                0x01
#define LCD_CTRL                0x00
                                
#define LCD_C_BLINK_ON          0x09
#define LCD_C_BLINK_OFF         0xfe
#define LCD_C_ON                0x0a
#define LCD_C_OFF               0xfd
#define LCD_DISPLAY_ON          0x0c
#define LCD_DISPLAY_OFF         0xfb
#define LCD_BRIGHT_25           0x23
#define LCD_BRIGHT_50           0x22
#define LCD_BRIGHT_75           0x21
#define LCD_BRIGHT_100          0x20
#define LCD_BACKLIGHT_ON        0x24
#define LCD_BACKLIGHT_OFF       0x25

// Key
#define KEY_PIN                 PINB
#define KEY_DDR                 DDRB
#define KEY_PORT                PORTB
                                
// Keypad
#define ROW1                    0x02
#define ROW2		                0x04
#define ROW3 		                0x08
#define ROW4		                0x10
#define COL1 		                0x20
#define COL2		                0x40
#define COL3		                0x80
#define COL4		                0x80

#define RESET                   20

//*****************************************************************************
//           C O N S T A N T			 				               
//*****************************************************************************
// SineTable
const ushort SineTable[256] =	 {133,136,139,142,
	  		   					   	        145,149,152,155,
									              158,161,164,167,
									              169,172,175,178,
									              181,184,186,189,
                  						  192,194,197,200,
                  						  202,205,207,209,
                  						  212,214,216,218,
                  						  220,222,224,226,
                  						  228,230,232,233,
                  						  235,237,238,240,
                  						  241,242,243,245,
                  						  246,247,248,248,
                  						  249,250,251,251,
                  						  252,252,252,253,
                  						  253,253,253,253,
                  						  253,253,252,252,
                  						  252,251,251,250,
                  						  249,248,248,247,
                  						  246,245,243,242,
                  						  241,240,238,237,
                  						  235,233,232,230,
                  						  228,226,224,222,
                  						  220,218,216,214,
                  						  212,209,207,205,
                  						  202,200,197,194,
                  						  192,189,186,184,
                  						  181,178,175,172,
                  						  169,167,164,161,
                  						  158,155,152,149,
                  						  145,142,139,136,
                  						  133,130,127,124,
                  						  121,118,115,112,
                  						  109,105,102,99,
                  						  96,93,90,87,
                  						  85,82,79,76,
                  						  73,70,68,65,
                  						  62,60,57,54,
                  						  52,49,47,45,
                  						  42,40,38,36,
                  						  34,32,30,28,
                  						  26,24,22,21,
                  						  19,17,16,14,
                  						  13,12,11,9,
                  						  8,7,6,6,
                  						  5,4,3,3,
                  						  2,2,2,1,
                  						  1,1,1,1,
                  						  1,1,2,2,
                  						  2,3,3,4,
                  						  5,6,6,7,
                  						  8,9,11,12,
                  						  13,14,16,17,
                  						  19,21,22,24,
                  						  26,28,30,32,
                  						  34,36,38,40,
                  						  42,45,47,49,
                  						  52,54,57,60,
                  						  62,65,68,70,
                  						  73,76,79,82,
                  						  85,87,90,93,
                  						  96,99,102,105,
                  						  109,112,115,118,
                  						  121,124,127,127};
	
//*****************************************************************************
//                            P R O T O T Y P E
//*****************************************************************************
void main(void);
void DoVolume(void);
void DoKey(ushort ReGen);
void DisplayInterComState(void);
void SetKeySoundLen(void);
void SetKeySoundTone(void);
void SetKeySoundVolume(void);
void SetKeySound(void);
void SetBroadCastEnable(void);
void SetStationAddress(void);
void StationCall(ushort Station);
void BroadCastCall(void);
void SendRelease(void);
void _StackOverflowed(char c);
void Led(void);
void Info(void);
void Delay(ushort Del);

// A/D
void ADInit(void);
void ADStart(void);
void ADDone(void);

// Voice
void VoiceEnginInit(void);
void VoiceEngin(void);
void VoiceVolume(ushort Vol);

// Network
void NetInit(void);
void NetSendCommand(ushort To,ushort Command);
void NetDirectWriteByte(ushort Byte);
void NetWriteByte(ushort Byte);
ushort NetReadByte(void);
ushort NetGetInByteCount(void);
void NetRxByte(void);
void NetTxByte(void);
void NetGetAddress(void);

// LCD
void LCDInit(ushort x,ushort y);
void LCDOff(void);
void LCDClrSCR(void);
void LCDGotoXY(ushort,ushort);
ushort LCDWhereX(void);
ushort LCDWhereY(void);
void LCDWriteString(char *);
void LCDWriteConstString(const char *);
void LCDWriteChar(char);
void LCDWriteData(ushort,ushort);
ushort LCDReadData(ushort rs);
void LCDCreateCHR(ushort chnumber, char *ptr);  
void LCDWait(void);
void LCDDelay50us(int Delay);
void LCDTextAttr(ushort attribute);
void LCDContrast(ushort Cont);

// Key
void KeyInit(void);
ushort KeyLook(void);
void KeyScan(void);

//*****************************************************************************
//                      G L O B A L   V A R I A B L E
//*****************************************************************************

// LCD
ushort MaxX, MaxY, LCDStat;
char Text[20];

// Keypad
ushort Key;
ushort KeySound;
ushort KeySoundVolume;
ushort KeySoundTone;
ushort KeySoundLen;

// Network
ushort  NetAddress;
ushort DataInBuffer[VOICEBUFFER];
ushort *DataInBufferWrite;
ushort *DataInBufferRead;
uint DataInBufferQte;

ushort DataOutBuffer[VOICEBUFFER];
ushort *DataOutBufferWrite;
ushort *DataOutBufferRead;
uint DataOutBufferQte;

ushort InterCommMode;
ushort IncommingCall;
ushort Volume = 0x80;
ushort Tone = 8;

// Others
ushort BroadCastEnable;
ushort RefreshDisplay;

//*****************************************************************************
//                      M A I N
//*****************************************************************************
void main()
{
  WDR();
  WDTCR = 0x0f; // Watch Dog enable
  LCDInit(20,2);
  KeyInit();
  NetInit();
  ADInit();
  VoiceEnginInit();

  SEI(); //re-enable interrupts

  EEPROM_READ(EEPROM_VOLUME, Volume);
  EEPROM_READ(EEPROM_KEYSOUND, KeySound);
  EEPROM_READ(EEPROM_KEYSOUNDVOLUME, KeySoundVolume);
  EEPROM_READ(EEPROM_KEYSOUNDTONE, KeySoundTone);
  EEPROM_READ(EEPROM_KEYSOUNDLEN, KeySoundLen);
  EEPROM_READ(EEPROM_BROADCAST, BroadCastEnable);
  
  if (KeySoundTone > 59) KeySoundTone = 59;
  if (KeySoundLen > 10) KeySoundLen = 10;

  Info();
  InterCommMode = TONE;
  VoiceVolume(0x40);  
  Delay(50);
  LCDClrSCR();
  VoiceVolume(0);
  InterCommMode = OFF;
  IncommingCall = FALSE;
  DoKey(TRUE);

  while(1)
  {
    WDR();
    _StackCheck();
    
    if (KeyLook() == RESET) while(TRUE);
    if ((!IncommingCall) && (InterCommMode != BUSY)) DoKey(FALSE);
    if (IncommingCall) DoVolume();
    if (InterCommMode == BUSY) DisplayInterComState();
  }
}

/******************************************************************************

Name:			void DoKey(void)

Description:	

Input:			none

Output:			none

Misc:			

******************************************************************************/
void DoVolume(void)
{
  static int Ticker,VolumeChange=FALSE;

  DisplayInterComState();

  Ticker++;
  Key = KeyLook();

  if ((Key == 11) && (Volume < 255)) Volume++;
  if ((Key == 10) &&(Volume > 1)) Volume--;

  if (Key != 0xff)
  {
    if (Key == 8) 
    {
    	InterCommMode = OFF;
  		IncommingCall = FALSE;
    }
    Ticker = 0;
    VoiceVolume(Volume);
    VolumeChange = TRUE;
    LCDGotoXY(1,2);
    LCDWriteConstString("                    \0");
    LCDGotoXY(1,2);
    csprintf(&Text[0],"    Volume :%d  \0",Volume);
    LCDWriteString(&Text[0]);
    Delay(1);
    Key = 0xff;
  }

  if ((VolumeChange == TRUE) && (Ticker > 1000))
  {
    EEPROM_WRITE(EEPROM_VOLUME, Volume);
    VolumeChange = FALSE;
  }
}

/******************************************************************************

Name:			void DoKey(void)

Description:	

Input:			none

Output:			none

Misc:			

******************************************************************************/
void DoKey(ushort ReGen)
{
  static ushort KeyFirst,KeySecond,Value,Update;

  Update = FALSE;

  DisplayInterComState();

  if (ReGen)
  {
    LCDClrSCR();
    LCDGotoXY(1,1);
    LCDWriteConstString(VERSIONTEXT);
    LCDGotoXY(1,2);
    LCDWriteData(LCD_DATA,KeySecond+48);
    LCDWriteData(LCD_DATA,KeyFirst+48);
    Value = (KeySecond * 10) + KeyFirst;
    Update = TRUE;
  }
  else
  {
    KeyScan();

    if ((Key != 0xff) && (Key != 10) && (Key != 11))
    {
      KeySecond = KeyFirst;
      KeyFirst = Key;
      LCDGotoXY(1,2);
      LCDWriteData(LCD_DATA,KeySecond+48);
      LCDWriteData(LCD_DATA,KeyFirst+48);
      Value = (KeySecond * 10) + KeyFirst;
      Update = TRUE;
      Key = 0xff;
    }
  }

//--------------------

  if (Update == TRUE)
  {
    LCDGotoXY(3,2);
    if (Value == 0) LCDWriteConstString("->Broadcast Call #\0");
    else if ((Value >= 1) && (Value < 40)) LCDWriteConstString("->Calling #       \0");
    else if (Value == 88) LCDWriteConstString("->Send Release #  \0");
    else if (Value == 99) LCDWriteConstString("->Set Address #   \0");
    else if (Value == 98) LCDWriteConstString("->Set Key Sound # \0");
    else if (Value == 97) LCDWriteConstString("->Set Key Tone # \0 ");
    else if (Value == 96) LCDWriteConstString("->Set Key Volume #\0");
    else if (Value == 95) LCDWriteConstString("->Set Tone Len # \0");
    else if (Value == 94) LCDWriteConstString("->Set BroadCast # \0");
    else if (Value == 93) LCDWriteConstString("->Reset Station # \0");
    else if ((Value >= 40) && (Value < 94)) LCDWriteConstString("->                \0");
  }

//--------------------

  if (Key == 11)
  {
    Key = 0xff;
    if (Value == 0) BroadCastCall();
    else if ((Value >= 0) && (Value < 40)) StationCall(Value);
    else if (Value == 88) SendRelease();
    else if (Value == 99) SetStationAddress();
    else if (Value == 98) SetKeySound();
    else if (Value == 97) SetKeySoundTone();
    else if (Value == 96) SetKeySoundVolume();
    else if (Value == 95) SetKeySoundLen();
    else if (Value == 94) SetBroadCastEnable();
  	else if (Value == 93) {CLI(); while(TRUE);}
  }
}

/******************************************************************************

Name:			  void DisplayInterComState(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void DisplayInterComState(void)
{
   static ushort LastIntercommMode;

   if (LastIntercommMode != InterCommMode)
   {
      LastIntercommMode = InterCommMode;
      if (LastIntercommMode == OFF)
      {
         LCDClrSCR();
         LCDWriteConstString("Communication ended \0"); 
         Delay(75);
         LCDGotoXY(1,1);
         LCDWriteConstString("                    \0"); 
	 			DoKey(TRUE);
      }
     if (LastIntercommMode == BUSY)
      {
         LCDClrSCR();
         LCDWriteConstString(" --    B u s y   -- \0");
         LCDGotoXY(1,2);
         LCDWriteConstString(" --  Please Wait -- \0");
      }
     if (LastIntercommMode == TALK)
      {
         LCDClrSCR();
         LCDWriteConstString(" --Incomming Call-- \0");
         LCDGotoXY(1,2);
         csprintf(&Text[0],"    Volume :%d  \0",Volume);
         LCDWriteString(&Text[0]);
      }
   }
}

/******************************************************************************

Name:			    SetKeySoundLen(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetKeySoundLen(void)
{
  static ushort KeyFirst,KeySecond,Value;

  LCDClrSCR();
  LCDGotoXY(1,1);
  csprintf(&Text[0],"Current Tone : %d\0",KeySoundLen);
  LCDWriteString(&Text[0]);
  LCDGotoXY(1,2);
  LCDWriteConstString("New Tone(00-10):00\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key != 10) && (Key != 11))
    {
      KeySecond = KeyFirst;
      KeyFirst = Key;
      LCDGotoXY(17,2);
      LCDWriteData(LCD_DATA,KeySecond+48);
      LCDWriteData(LCD_DATA,KeyFirst+48);
      Value = (KeySecond * 10) + KeyFirst;
      if (Value < 11) KeySoundLen = Value;
      Key = 0xff;
    }

    if (Key == 11)
    {
      Key = 0xff;
      if ((Value >= 0) && (Value < 11))
      {
        EEPROM_WRITE(EEPROM_KEYSOUNDLEN, Value);
        LCDGotoXY(1,1);
        LCDWriteConstString("  New Value Saved   \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString("  ---------------   \0");
        DoKey(TRUE);
        return;
      }
      else
      {
        LCDGotoXY(1,1);
        LCDWriteConstString(" Value Out Of Range \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString(" ------------------ \0");
        DoKey(TRUE);
        return;
      }
    }
  }  
}

/******************************************************************************

Name:			    SetKeySoundTone(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetKeySoundTone(void)
{
  static ushort KeyFirst,KeySecond,Value;

  LCDClrSCR();
  LCDGotoXY(1,1);
  csprintf(&Text[0],"Current Tone : %d\0",KeySoundTone);
  LCDWriteString(&Text[0]);
  LCDGotoXY(1,2);
  LCDWriteConstString("New Tone(00-59):00\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key != 10) && (Key != 11))
    {
      KeySecond = KeyFirst;
      KeyFirst = Key;
      LCDGotoXY(17,2);
      LCDWriteData(LCD_DATA,KeySecond+48);
      LCDWriteData(LCD_DATA,KeyFirst+48);
      Value = (KeySecond * 10) + KeyFirst;
      if (Value < 59) KeySoundTone = Value;
      Key = 0xff;
    }

    if (Key == 11)
    {
      Key = 0xff;
      if ((Value >= 0) && (Value <59))
      {
        EEPROM_WRITE(EEPROM_KEYSOUNDTONE, Value);
        LCDGotoXY(1,1);
        LCDWriteConstString("  New Value Saved   \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString("  ---------------   \0");
        DoKey(TRUE);
        return;
      }
      else
      {
        LCDGotoXY(1,1);
        LCDWriteConstString(" Value Out Of Range \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString(" ------------------ \0");
        DoKey(TRUE);
        return;
      }
    }
  }  
}

/******************************************************************************

Name:			    SetKeySoundVolume(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetKeySoundVolume(void)
{
  static ushort KeyFirst,KeySecond,Value;

  LCDClrSCR();
  LCDGotoXY(1,1);
  csprintf(&Text[0],"Current Key Vol:%d\0",KeySoundVolume);
  LCDWriteString(&Text[0]);
  LCDGotoXY(1,2);
  LCDWriteConstString("New Key Vol 00-99:00\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key != 10) && (Key != 11))
    {
      KeySecond = KeyFirst;
      KeyFirst = Key;
      LCDGotoXY(19,2);
      LCDWriteData(LCD_DATA,KeySecond+48);
      LCDWriteData(LCD_DATA,KeyFirst+48);
      Value = (KeySecond * 10) + KeyFirst;
      KeySoundVolume = Value;
      Key = 0xff;
    }

    if (Key == 11)
    {
      Key = 0xff;
      EEPROM_WRITE(EEPROM_KEYSOUNDVOLUME, Value);
      LCDGotoXY(1,1);
      LCDWriteConstString("  New Value Saved   \0");
      Delay(75);
      LCDGotoXY(1,1);
      LCDWriteConstString("  ---------------   \0");
      DoKey(TRUE);
      return;
    }
  }
}

/******************************************************************************

Name:			    SetKeySound(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetKeySound(void)
{
  LCDClrSCR();
  LCDGotoXY(1,1);
  if (KeySound) LCDWriteConstString("Key Sound is ON     \0");
  else LCDWriteConstString("Key Sound is OFF    \0");
  LCDGotoXY(1,2);
  LCDWriteConstString("* To Toggle  #->Save\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key == 10))
    {
      if (KeySound) KeySound = 0;
      else KeySound = 1;
      Key = 0xff;
      LCDGotoXY(1,1);
      if (KeySound) LCDWriteConstString("Key Sound is ON     \0");
      else LCDWriteConstString("Key Sound is OFF    \0");
    }

    if (Key == 11)
    {
      Key = 0xff;
      EEPROM_WRITE(EEPROM_KEYSOUND, KeySound);
      LCDGotoXY(1,1);
      LCDWriteConstString("  New Value Saved   \0");
      Delay(75);
      LCDGotoXY(1,1);
      LCDWriteConstString("  ---------------   \0");
      DoKey(TRUE);
      return;
    }
  }
}

/******************************************************************************

Name:			    SetBroadCastEnable(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetBroadCastEnable(void)
{
  LCDClrSCR();
  LCDGotoXY(1,1);
  if (BroadCastEnable) LCDWriteConstString("BroadCast is ON     \0");
  else LCDWriteConstString("BroadCast is OFF    \0");
  LCDGotoXY(1,2);
  LCDWriteConstString("* To Toggle  #->Save\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key == 10))
    {
      if (BroadCastEnable) BroadCastEnable = 0;
      else BroadCastEnable = 1;
      Key = 0xff;
      LCDGotoXY(1,1);
      if (BroadCastEnable) LCDWriteConstString("BroadCast is ON     \0");
      else LCDWriteConstString("BroadCast is OFF    \0");
    }

    if (Key == 11)
    {
      Key = 0xff;
      EEPROM_WRITE(EEPROM_BROADCAST, BroadCastEnable);
      LCDGotoXY(1,1);
      LCDWriteConstString("  New Value Saved   \0");
      Delay(75);
      LCDGotoXY(1,1);
      LCDWriteConstString("  ---------------   \0");
      DoKey(TRUE);
      return;
    }
  }
}

/******************************************************************************

Name:			    SetStationAddress(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void SetStationAddress(void)
{
  static ushort KeyFirst,KeySecond,Value;

  LCDClrSCR();
  LCDGotoXY(1,1);
  csprintf(&Text[0],"Current Address : %d\0",NetAddress);
  LCDWriteString(&Text[0]);
  LCDGotoXY(1,2);
  LCDWriteConstString("New Adr(00-39) : 00\0");

  while(TRUE)
  {
    WDR();
    KeyScan();
    if ((Key != 0xff) && (Key != 10) && (Key != 11))
    {
      KeySecond = KeyFirst;
      KeyFirst = Key;
      LCDGotoXY(18,2);
      LCDWriteData(LCD_DATA,KeySecond+48);
      LCDWriteData(LCD_DATA,KeyFirst+48);
      Value = (KeySecond * 10) + KeyFirst;
      Key = 0xff;
    }

    if (Key == 11)
    {
      Key = 0xff;
      if ((Value >= 0) && (Value <40))
      {
        EEPROM_WRITE(EEPROM_ADDRESS, Value);
        NetGetAddress();
        LCDGotoXY(1,1);
        LCDWriteConstString("  New Value Saved   \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString("  ---------------   \0");
        DoKey(TRUE);
        return;
      }
      else
      {
        LCDGotoXY(1,1);
        LCDWriteConstString(" Value Out Of Range \0");
        Delay(75);
        LCDGotoXY(1,1);
        LCDWriteConstString(" ------------------ \0");
        DoKey(TRUE);
        return;
      }
    }
  }
}

/******************************************************************************

Name:			    void StationCall(ushort Station)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void StationCall(ushort Station)
{  
  VoiceVolume(Volume); 
  LCDClrSCR();
  LCDGotoXY(1,1);
  csprintf(&Text[0],"Calling Station :#%d\0",Station); 
  LCDWriteString(&Text[0]);
  LCDGotoXY(1,2);
  LCDWriteConstString("#->Talk *->End      \0"); 
  Delay(10);

  NetSendCommand(0xff,BUSY);
  NetSendCommand(Station,TALK);
  InterCommMode = LISTEN;
  NET_UCSRA &= ~(1<<MPCM);
  
  while(KeyLook() != 10)
  {
    WDR();
    if ((InterCommMode == LISTEN) && (KeyLook() == 11))
      {
        NetSendCommand(Station,LISTEN);
        Delay(2);
        InterCommMode = TALK;
      }
    if ((InterCommMode == TALK) && (KeyLook() == 0xff))
      {
        NetSendCommand(Station,TALK);
        Delay(2);
        InterCommMode = LISTEN;
        NET_UCSRA &= ~(1<<MPCM);
      }
  }

	SendRelease();
	DoKey(TRUE);
}        

/******************************************************************************

Name:			    void BroadCastCall(void)

Description:	

Input:			  none

Output:			  none

Misc:			

******************************************************************************/
void BroadCastCall(void)
{  
  VoiceVolume(Volume); 
  LCDClrSCR();
  LCDGotoXY(1,1);
  LCDWriteConstString("   BroadCast Call   \0"); 
  LCDGotoXY(1,2);
  LCDWriteConstString("   #->Talk *->End   \0"); 
  Delay(10);

  NetSendCommand(0xff,BUSY);
  InterCommMode = OFF;
  
  while(KeyLook() != 10)
  {
    WDR();
    if ((InterCommMode == OFF) && (KeyLook() == 11))
      {
        NetSendCommand(0xfe,LISTEN);
        InterCommMode = TALK;
      }
  }
  SendRelease();
	DoKey(TRUE);
}        

/******************************************************************************

Name:         void SendRelease(void)

Description:  This function release the communication



Input:        none

Output:       none

Misc:
******************************************************************************/
void SendRelease(void)
{
  InterCommMode = OFF;
  IncommingCall = FALSE;
  Delay(10);
  NetSendCommand(0xff,OFF);
  Delay(10);
  NetSendCommand(0xff,OFF);
  Delay(10);
  NetSendCommand(0xff,OFF);
  VoiceVolume(0); 
  LCDGotoXY(1,1);
  LCDWriteConstString("Communication ended \0"); 
  Delay(75);
  LCDGotoXY(1,1);
  LCDWriteConstString("                    \0"); 
  DoKey(TRUE);
}

/******************************************************************************

Name:         void _StackOverflowed(char c)

Description:  This function is automaticaly called if
              the stack crash/


Input:        none

Output:       none

Misc:
******************************************************************************/
void _StackOverflowed(char c)
{
  while(TRUE)
  {
    DDRD |= LED;
    PORTD |= LED;
  }
}

/******************************************************************************

Name:         void Led(void)

Description:  This function invert the led

Input:        none

Output:       none

Misc:
******************************************************************************/
void Led(void)
{
  DDRD |= LED;
  PORTD ^= LED;
}

/*****************************************************************************

Name:         void Info(void)

Description:  Display power on message

Input:        void

Output:       void

Misc:         

******************************************************************************/
void Info(void)
{
  LCDClrSCR();
  LCDGotoXY(1,1);
  LCDWriteConstString(VERSIONTEXT);
  LCDGotoXY(1,2);
  LCDWriteConstString("--------------------\0");
}

/******************************************************************************

Name:         void Delay(ushort Del)

Description:  This function make a delay

Input:        none

Output:       none

Misc:
******************************************************************************/
void Delay(ushort Del)
{
  int i,j;
  for (i=0;i<Del;i++) for (j=0;j<32000;j++) WDR();
}

/******************************************************************************

Name:         void ADInit(void)

Description:  This function initialize the AD converter

Input:        none

Output:       none

Misc:
******************************************************************************/
void ADInit(void)
{
  DDRA &= ~(1<<PA1);
  PORTA &= ~(1<<PA1);
  ADMUX = (1<<REFS0) + (1<<ADLAR) + (1<<MUX0);
  ADCSRA = (1<<ADEN) + (1<<ADIE) + (1<<ADPS2) + (1<<ADPS0);  // 500ks/sec
}

/******************************************************************************

Name:         void ADStart(void)

Description:  This function start a AD convertion

Input:        none

Output:       none

Misc:
******************************************************************************/
void ADStart(void)
{
  ADCSRA |= (1<<ADSC);
}

/******************************************************************************

Name:         void ADDone(void)

Description:  This function is automaticaly called when the AD had finish
              is convertion

Input:        none

Output:       none

Misc:
******************************************************************************/
#pragma interrupt_handler ADDone:17
void ADDone(void)
{
  if (InterCommMode != OFF) NetWriteByte(ADCH);
}

/******************************************************************************

Name:         void VoiceEnginInit(void)

Description:  This function initialize the voice digitalizer and reproducer

Input:        none

Output:       none

Misc:
******************************************************************************/
void VoiceEnginInit(void)
{
  DataOutBufferRead = &DataOutBuffer[0];
  DataOutBufferWrite = &DataOutBuffer[0];
  DataOutBufferQte = 0;

  DataInBufferRead = &DataInBuffer[0];
  DataInBufferWrite = &DataInBuffer[0];
  DataInBufferQte = 0;
  
  InterCommMode = OFF;
  
  TCCR0 = 0x00; //stop
  ASSR  = 0x00; //set async mode
  TCNT0 = 0x9C; //set count 20k
  OCR0  = 0x64;
  OCR1AL = 0x7f;
  OCR1BL = 0xff;
  TCCR0 = 0x02; //start timer for 8khz sampling
  TIMSK |= 0x01; //timer interrupt sources

  TCCR1A = (1<<COM1A1) + (1<<COM1B1) + (1<<WGM10);
  TCCR1B = (1<<WGM12) + (1<<CS10);
  DDRD |= PWM_VOICE + PWM_VOLUME + AMPLI_STANDBY;
  PORTD &= ~AMPLI_STANDBY;
}

/******************************************************************************

Name:         void DataEngin(void)

Description:  This function analyse data stream

Input:        none

Output:       none

Misc:
******************************************************************************/
#pragma interrupt_handler VoiceEngin:12
void VoiceEngin(void)
{
  static ushort i = 0;
  TCNT0 = 0x9c; //reload counter value 20k

  if (InterCommMode == TALK) ADStart();

  if (InterCommMode == LISTEN)
  {
    if (NetGetInByteCount() != 0) OCR1AL = NetReadByte();
  }
  else if (InterCommMode == TONE) OCR1AL = SineTable[i+=Tone];
  else OCR1AL = 0x7f;
}

/******************************************************************************

Name:         void VoiceVolume(ushort Vol)

Description:  This function set the output volume

Input:        ushort -> Volume

Output:       none

Misc:
******************************************************************************/
void VoiceVolume(ushort Vol)
{
  if (Vol == 0) PORTD &= ~AMPLI_STANDBY;
  else PORTD |= AMPLI_STANDBY;
  OCR1BL = Vol;
}

//*****************************************************************************
//        N E T W O R K     F U N C T I O N
//*****************************************************************************
/******************************************************************************

Name:         void NetInit(void)

Description:  This function initialize for NET network

Input:        None

Output:       None

Misc:
******************************************************************************/
void NetInit(void)
{
  NET_UBRRH = 0x00;                               //set baud rate hi
  NET_UBRRL = (XTAL / (16 * NET_SPEED)) - 1;      //set baud rate
  NET_UCSRA = (1<<MPCM);
  NET_UCSRB = (1<<UCSZ2) + (1<<RXCIE)+(1<<RXEN)+(1<<TXEN);
  NET_UCSRC = (1<<URSEL) + (1<<UCSZ1) + (1<<UCSZ0);
  NET_DDR |= NET_TX;
  NetGetAddress();
}

/******************************************************************************

Name:         void NetSendCommand(ushort To,ushort Command)

Description:  This send a command to another node


Input:        ushort To
              ushort Command

Output:       

Misc:
******************************************************************************/
void NetSendCommand(ushort To,ushort Command)
{
  InterCommMode = OFF;
  NET_UCSRA |= (1<<MPCM);
  DataInBufferQte = 0;
  DataOutBufferQte = 0;
  while (!(NET_UCSRA & (1 << UDRE))) WDR();  // Wait for empty transmit buffer
  NET_UCSRB |= (1<<TXB8);
  NetDirectWriteByte(To);
  NET_UCSRB &= ~(1<<TXB8);
  NetDirectWriteByte(Command);
}

/******************************************************************************

Name:         void NetDirectWriteByte(ushort Byte)

Description:  This function write a byte directly in the serial port


Input:        none

Output:       

Misc:
******************************************************************************/
void NetDirectWriteByte(ushort Byte)
{
  NET_UDR = Byte;
  while (!(NET_UCSRA & (1 << UDRE))) WDR();  // Wait for empty transmit buffer
}

/******************************************************************************

Name:         void NetWriteByte(ushort Byte)

Description:  This function write a byte in the TX buffer


Input:        none

Output:       

Misc:
******************************************************************************/
void NetWriteByte(ushort Byte)
{
  if (DataOutBufferQte++ > VOICEBUFFER)
  {
    DataOutBufferQte = 0;
    DataOutBufferWrite = &DataOutBuffer[0];
    DataOutBufferRead = &DataOutBuffer[0];
  }

  *DataOutBufferWrite++ = Byte;
  if (DataOutBufferWrite > &DataOutBuffer[VOICEBUFFER-1])
  {
    DataOutBufferWrite = &DataOutBuffer[0];
  }
  NET_UCSRB |= (1<<UDRIE);
}

/******************************************************************************

Name:         ushort NetReadByte(void)

Description:  This function read a byte in the RX buffer


Input:        none

Output:       

Misc:
******************************************************************************/
ushort NetReadByte(void)
{
  ushort Byte = 0x00;

  if (DataInBufferQte != 0)
  {
    DataInBufferQte--;
    Byte = *DataInBufferRead++;
    if (DataInBufferRead > &DataInBuffer[VOICEBUFFER-1])
    {
      DataInBufferRead = &DataInBuffer[0];
    }
  }
  return Byte;
}

/******************************************************************************

Name:         ushort NetGetInByteCount(void)

Description:  This function return the Qte of byte in the RX buffer


Input:        none

Output:       

Misc:
******************************************************************************/
ushort NetGetInByteCount(void)
{
  return DataInBufferQte; 
}

/******************************************************************************

Name:         void NetRxByte(void)

Description:  This function is automaticaly called each
              time a char is received on the NET bus.


Input:        none

Output:       

Misc:
******************************************************************************/
#pragma interrupt_handler NetRxByte:14
void NetRxByte(void)
{
  static ushort InByte;
  static ushort Command = FALSE;

  if (NET_UCSRA & ((1<<FE)+(1<<DOR)))
  {
    InByte = NET_UDR;
    return;
  }
  
  if (NET_UCSRB & 0x02)
  {
    InByte = NET_UDR;
    if ((InByte == NetAddress) || ((InByte == 0xfe) && (BroadCastEnable == TRUE)) || (InByte == 0xff))
    {
      NET_UCSRA &= ~(1<<MPCM);
      Command = TRUE;
      return;
    }
    else
    {
      NET_UCSRA |= (1<<MPCM);
      Command = FALSE;
      return;
    }
  }

  else if (Command == TRUE)
  {
    Command = FALSE;
    InByte = NET_UDR;

    if (InByte == OFF)
    {
      InterCommMode = OFF;
      IncommingCall = FALSE;
      VoiceVolume(0);
    }

    else if (InByte == TALK)
    {
      IncommingCall = TRUE;
      InterCommMode = TALK;
    }

    else if (InByte == LISTEN)
    {
      VoiceVolume(Volume);
      IncommingCall = TRUE;
      InterCommMode = LISTEN;
    }

    else if (InByte == BUSY)
    {
      InterCommMode = BUSY;
    }
    return;
  }

  if (DataInBufferQte++ > VOICEBUFFER)
  {
    DataInBufferQte = 0;
    DataInBufferWrite = &DataInBuffer[0];
    DataInBufferRead = &DataInBuffer[0];
  }
  *DataInBufferWrite++ = NET_UDR;;

  if (DataInBufferWrite > &DataInBuffer[VOICEBUFFER-1])
  {
    DataInBufferWrite = &DataInBuffer[0];
  }
}

/******************************************************************************

Name:         void NetTxByte(void)

Description:  This function is automaticaly called each
              time a char had been send on the NET bus.


Input:        none

Output:       

Misc:
******************************************************************************/
#pragma interrupt_handler NetTxByte:15
void NetTxByte(void)
{
  if (DataOutBufferQte == 0) NET_UCSRB &= ~(1<<UDRIE); /* TX int empty disable */
  else
  {
    DataOutBufferQte--;
    NET_UDR = *DataOutBufferRead++;
    if (DataOutBufferRead > &DataOutBuffer[VOICEBUFFER-1])
    {
      DataOutBufferRead = &DataOutBuffer[0];
    }
  }
}

/******************************************************************************

Name:         void NetGetAddress(void)

Description:  Get the network address of this node
              from dip switch

Input:        None

Output:       None

Misc:

******************************************************************************/
void NetGetAddress(void)
{
  NetAddress = NET_GET_ADDRESS;
}

//*****************************************************************************
// LCD Code
//*****************************************************************************
/******************************************************************************

Name:         LCDCreateCHR

Description:  Create a new char in CGRAM

Input:        ushort  : Character number (0-7)
              *ushort : 7 ushort containing the data

Output:       none

Misc:

******************************************************************************/
void LCDCreateCHR(ushort chnumber, char *ptr)
{
  ushort i,j;

  i = 8 * chnumber;

  for (j=0;j<8;j++)
  {
    LCDWriteData(LCD_CTRL,(0x40 | (i + j)));
    LCDWriteData(LCD_DATA,ptr[j]);
  }
}


/******************************************************************************

Name:         void LCDClrSCR(void)

Description:  Clear the LCD

Input:        none

Output:       none

Misc:

******************************************************************************/
void LCDClrSCR(void)
{
  LCDWait();
  LCDWriteData(LCD_CTRL,0x01);      // Clear display
}

/******************************************************************************

Name:         LCDGotoXY(ushort x, ushort y)

Description:  Position cursor on the LCD at X & Y location

Input:        X -> X position on the LCD
              Y -> Y position on the LCD

Output:       none

Misc:

******************************************************************************/
void LCDGotoXY(ushort x,ushort y)
{
  ushort address;

  x--;

  if (MaxY < 3)
  {
    switch(y)
    {
      case '\x01' :
      address = 0 + x;
      break;
      case '\x02' :
      address = 64 + x;
      break;
    }
  }
  else
  {
    switch(y)
    {
      case '\x01' :
      address = 0 + x;
      break;
      case '\x02' :
      address = 64 + x;
      break;
      case '\x03' :
      address = 20 + x;
      break;
      case '\x04' :
      address = 84 + x;
      break;
    }
  }
  LCDWriteData(LCD_CTRL,address | 0x80);
}

/******************************************************************************

Name:         LCDWhereX

Description:  Get the x position of the cursor on the LCD

Input:        None

Output:       ushort : Cursor position "x"

Misc:

******************************************************************************/
ushort LCDWhereX(void)
{
ushort i,x,y;

i = LCDReadData(LCD_CTRL) & 0x7f;

y = LCDWhereY();

if (MaxY == 2)
    {
    if (y == 1) x = i;
    if (y == 2) x = i - 0x40;
    }
else if (MaxY == 4)
    {
    if (y == 1) x = i;
    if (y == 2) x = i - 0x40;
    if (y == 3) x = i - 0x14;
    if (y == 4) x = i - 0x54;
    }
x = x + 1;
return x;
}

/******************************************************************************

Name:         LCDWhereY

Description:  Get the y position of the cursor on the LCD

Input:        None

Output:       ushort : Cursor position "y"

Misc:

******************************************************************************/
ushort LCDWhereY(void)
{
  ushort i,j;

  i = LCDReadData(LCD_CTRL) & 0x7f;

  if (MaxY == 2)
  {
    if (i < 0x28) j = 1;
    else j = 2;
  }
  else if (MaxY == 4)
  {
    if ((i >0x00) && (i < 0x15)) j = 1;
    if ((i >0x14) && (i < 0x28)) j = 3;
    if ((i >0x40) && (i < 0x55)) j = 2;
    if ((i >0x54) && (i < 0x68)) j = 4;
  }
  return j;
}

/******************************************************************************

Name:         WtireByteLCD(char byte)

Description:  Write a byte on the LCD at cursor position

Input:        byte

Output:       none

Misc:

******************************************************************************/
void LCDWriteChar(char byte)
{
  char tmp;

  tmp = byte & 0xf0;
  tmp = tmp >> 4;
  tmp += 0x30;
  if (tmp > 0x39) tmp += 0x07;
  LCDWriteData(LCD_DATA,tmp);

  tmp = byte & 0x0f;
  tmp += 0x30;
  if (tmp > 0x39) tmp += 0x07;
  LCDWriteData(LCD_DATA,tmp);
}

/******************************************************************************

Name:         void LCDWriteString(char *ptr)

Description:  Write a string from RAM on the LCD

Input:        string pointer

Output:       none

Misc:

******************************************************************************/
void LCDWriteString(char *ptr)
{
  ushort i;

  for (i=1;i<41;i++)
  {
    if (*ptr == 0x00) break;
    LCDWriteData(LCD_DATA,*ptr++);
  }
}

/******************************************************************************

Name:         void LCDWriteConstString(const char *ptr)

Description:  Write a constant string on the LCD

Input:        string pointer

Output:       none

Misc:

******************************************************************************/
void LCDWriteConstString(const char *ptr)
{
  ushort i;

  for (i=1;i<41;i++)
  {
    if (*ptr == 0x00) break;
    else if (i == 21) LCDGotoXY(1,2);
    LCDWriteData(LCD_DATA,*ptr++);
  }
}

/******************************************************************************

Name:         void LCDWriteData(ushort rs, ushort ch)

Description:  Write a byte in rs of the LCD

Input:        rs -> Register select
              ch -> byte to write

Output:       none

Misc:

******************************************************************************/
void LCDWriteData(ushort rs,ushort ch)
{
  ushort Stat;
  
  Stat = LCD_PIN & LCD_BACKLIGHT;

  LCD_PORT = Stat;
  
  LCDWait();

  if ((ch & 0x80) == 0x80) LCD_PORT |= LCD_D7;
  if ((ch & 0x40) == 0x40) LCD_PORT |= LCD_D6;
  if ((ch & 0x20) == 0x20) LCD_PORT |= LCD_D5;
  if ((ch & 0x10) == 0x10) LCD_PORT |= LCD_D4;
  if (rs == 1) LCD_PORT |= LCD_RS;
	LCDDelay50us(1);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;

  LCD_PORT = Stat;
  if ((ch & 0x08) == 0x08) LCD_PORT |= LCD_D7;
  if ((ch & 0x04) == 0x04) LCD_PORT |= LCD_D6;
  if ((ch & 0x02) == 0x02) LCD_PORT |= LCD_D5;
  if ((ch & 0x01) == 0x01) LCD_PORT |= LCD_D4;
  if (rs == 1) LCD_PORT |= LCD_RS;
	LCDDelay50us(1);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
}

/******************************************************************************

Name:         ushort LCDRead(ushort rs)

Description:  read a byte in rs of the LCD 

Input:        rs -> Register select

Output:       ushort

Misc:     

******************************************************************************/ 
ushort LCDReadData(ushort rs)
{
  ushort Byte=0;

  LCD_DDR &= ~(LCD_D7 + LCD_D6 + LCD_D5 + LCD_D4);

  LCD_PORT = LCD_PIN & LCD_BACKLIGHT;
  LCD_PORT |= LCD_RW;
  if (rs == 1) LCD_PORT |= LCD_RS;
  LCDDelay50us(1);

  LCD_PORT |= LCD_E;
  LCD_PORT |= LCD_E;
  if (LCD_PIN & LCD_D7) Byte |= 0x80;
  if (LCD_PIN & LCD_D6) Byte |= 0x40;
  if (LCD_PIN & LCD_D5) Byte |= 0x20;
  if (LCD_PIN & LCD_D4) Byte |= 0x10;
  LCD_PORT &= ~LCD_E;

  LCD_PORT |= LCD_E;
  LCD_PORT |= LCD_E;
  if (LCD_PIN & LCD_D7) Byte |= 0x08;
  if (LCD_PIN & LCD_D6) Byte |= 0x04;
  if (LCD_PIN & LCD_D5) Byte |= 0x02;
  if (LCD_PIN & LCD_D4) Byte |= 0x01;
  LCD_PORT &= ~LCD_E;

  LCD_PORT &= ~LCD_RW;

  LCD_DDR |= LCD_D7 + LCD_D6 + LCD_D5 + LCD_D4;

  return (Byte);
}

/******************************************************************************

Name:         void LCDWait(void)

Description:  wait for the LCD to be ready

Input:        none

Output:       none

Misc:     

******************************************************************************/
void LCDWait(void)
{
  while((LCDReadData(0) & 0x80) == 0x80) WDR();
}

/******************************************************************************

Name:         void LCDDelay50us(int Delay)

Description:  Delay of 50 us with a 1Mhz resonator

Input:        Delay X x 50us

Output:       none

Misc:

******************************************************************************/
void LCDDelay50us(int Delay)
{
  int i,j;

  for (i=0;i<Delay;i++)
  {
    for (j=1;j<120;j++); 
    asm("WDR");
  }
}

/******************************************************************************

Name:         void LCDTextAttr(ushort Attribute)

Description:  Set the LCD attribute

Input:        LCD_C_BLINK_ON     : Cursor blink on
              LCD_C_BLINK_OFF    : Cursor blink off
              LCD_C_ON           : Cursor on
              LCD_C_OFF          : Cursor off
              LCD_DISPLAY_ON     : Display ON
              LCD_DISPLAY_OFF    : Display OFF
              LCD_BRIGHT_25      : VFD 25% Bright
              LCD_BRIGHT_50      : VFD 50% Bright
              LCD_BRIGHT_75      : VFD 75% Bright
              LCD_BRIGHT_100     : VFD 100% Bright
              LCD_BACK_LIGHT_ON  : Back Light ON
              LCD_BACK_LIGHT_OFF : Back Light OFF

Output:       none

Misc:

******************************************************************************/
void LCDTextAttr(ushort attribute)
{
switch (attribute)
    {
    case LCD_C_BLINK_ON:
         LCDStat |= LCD_C_BLINK_ON;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_C_BLINK_OFF:
         LCDStat &= LCD_C_BLINK_OFF;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_C_ON:
         LCDStat |= LCD_C_ON;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_C_OFF:
         LCDStat &= LCD_C_OFF;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_DISPLAY_ON:
         LCDStat |= LCD_DISPLAY_ON;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_DISPLAY_OFF:
         LCDStat &= LCD_DISPLAY_OFF;
         LCDWriteData(LCD_CTRL,LCDStat);
         break;
         
    case LCD_BRIGHT_25:
         LCDWriteData(LCD_CTRL,0x20);
         LCDWriteData(LCD_DATA,0x03);
         break;
         
    case LCD_BRIGHT_50:
         LCDWriteData(LCD_CTRL,0x20);
         LCDWriteData(LCD_DATA,0x02);
         break;
         
    case LCD_BRIGHT_75:
         LCDWriteData(LCD_CTRL,0x20);
         LCDWriteData(LCD_DATA,0x01);
         break;
         
    case LCD_BRIGHT_100:
         LCDWriteData(LCD_CTRL,0x20);
         LCDWriteData(LCD_DATA,0x00);
         break;    
              
    case LCD_BACKLIGHT_ON:
         LCD_PORT |= LCD_BACKLIGHT;
         break;
         
    case LCD_BACKLIGHT_OFF:
         LCD_PORT &= ~(LCD_BACKLIGHT);
         break;          
    }
}

/******************************************************************************

Name:         void LCDOn(ushort X, ushort Y)

Description:  Initialize LCD in 4bit mode

Input:        ushort X -> X size
              ushort Y -> Y size

Output:       none

Misc:

******************************************************************************/
void LCDInit(ushort X, ushort Y)
{
  MaxX = X;
  MaxY = Y;
  LCDStat = 0x0c;
  LCD_PORT = 0x00;
  LCD_DDR = LCD_D7+LCD_D6+LCD_D5+LCD_D4+LCD_E+LCD_RS+LCD_RW+LCD_BACKLIGHT;
  LCD_PORT = ~(LCD_D7+LCD_D6+LCD_D5+LCD_D4+LCD_E+LCD_RS+LCD_RW);
  LCD_PORT |= LCD_BACKLIGHT;
  LCDDelay50us(340);

  LCD_PORT = (LCD_D5 + LCD_D4 + LCD_BACKLIGHT);  
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);

  LCD_PORT = (LCD_D5 + LCD_BACKLIGHT);  
  LCD_PORT |= LCD_E;
  LCD_PORT &= ~LCD_E;
  LCDDelay50us(100);

  if (Y == 1) LCDWriteData(LCD_CTRL,0x20);    // 1 line
  else LCDWriteData(LCD_CTRL,0x28);           // 2 line

  LCDWriteData(LCD_CTRL,0x0c);                // Disp ON-Cur OFF-Blink OFF
  LCDWriteData(LCD_CTRL,0x01);                // Clear display
  LCDDelay50us(50);
  LCDWriteData(LCD_CTRL,0x06);                // Cursor INC Shift OFF

  DDRD |= 0x80;
  TCCR2 = 0x79;
  OCR2 = 0xff;
}

/******************************************************************************

Name:         void LCDContrast(ushort Cont)

Description:  Set the LCD Contrast

Input:        from 0 to 255

Output:       none

Misc:

******************************************************************************/
void LCDContrast(ushort Cont)
{
  OCR2 = Cont;
}

/******************************************************************************

Name:         void LCDOff(void)

Description:  Shut down LCD

Input:        none

Output:       none

Misc:

******************************************************************************/
void LCDOff(void)
{
 LCD_DDR = 0xff;
 LCD_PORT = 0x00; 
 DDRD &= ~0x80;
}

//*****************************************************************************
// Key Code
//*****************************************************************************

/******************************************************************************

Name:         void KeyInit(void)

Description:  Initialize IO's

Input:        none

Output:       none

Misc:

******************************************************************************/
void KeyInit(void)
{
  KEY_DDR |= ROW4 | ROW3 | ROW2 | ROW1;    // Row as output
  KEY_DDR &= ~(COL4 | COL3 | COL2 | COL1); // Col as input
  KEY_PORT |= COL4 | COL3 | COL2 | COL1;   // Col with pull up  
}

/******************************************************************************

Name:         void KeyScan(void)

Description:  ScanKeyPad

Input:        none

Output:       none

Misc:

******************************************************************************/
ushort KeyLook(void)
{
  ushort Val,i;
  
  Val = 0xff;
  
  KEY_PORT &= ~ROW1;
  for (i=0;i<255;i++) WDR();
  if (!(KEY_PIN & COL1)) Val = 1;
  if (!(KEY_PIN & COL2)) Val = 2;
  if (!(KEY_PIN & COL3)) Val = 3;
  KEY_PORT |= ROW1;

  KEY_PORT &= ~ROW2;
  for (i=0;i<255;i++) WDR();
  if (!(KEY_PIN & COL1)) Val = 4;
  if (!(KEY_PIN & COL2)) Val = 5;
  if (!(KEY_PIN & COL3)) Val = 6;
  KEY_PORT |= ROW2;

  KEY_PORT &= ~ROW3;
  for (i=0;i<255;i++) WDR();
  if (!(KEY_PIN & COL1)) Val = 7;
  if (!(KEY_PIN & COL2)) Val = 8;
  if (!(KEY_PIN & COL3)) Val = 9;
  KEY_PORT |= ROW3;
  
  KEY_PORT &= ~ROW4;
  for (i=0;i<255;i++) WDR();
  if (!(KEY_PIN & COL1)) Val = 10;
  if (!(KEY_PIN & COL2)) Val = 0;
  if (!(KEY_PIN & COL3)) Val = 11;
  if (!(KEY_PIN & (COL1 | COL3))) Val = 20;
  KEY_PORT |= ROW4;
  
  return Val;
}

/******************************************************************************

Name:         void KeyScan(void)

Description:  ScanKeyPad

Input:        none

Output:       none

Misc:

******************************************************************************/
void KeyScan(void)
{
  static unsigned char LastKey;
  unsigned char KeyRead;

  KeyRead = KeyLook();

  if ((KeyRead != LastKey) && (KeyRead != 0xff))
  {
    Key = KeyRead;
    if (KeySound)
    {
      Tone = KeySoundTone;
      VoiceVolume((KeySoundVolume*2)+55);
      InterCommMode = TONE;
      Delay(KeySoundLen);
      InterCommMode = OFF;
      VoiceVolume(0);
    }
  }
  LastKey = KeyRead;
}
